-- *************************************************************************
-- DISCLAIMER. THIS SOFTWARE WAS WRITTEN BY EMPLOYEES OF THE U.S.
-- GOVERNMENT AS A PART OF THEIR OFFICIAL DUTIES AND, THEREFORE, IS NOT
-- PROTECTED BY COPYRIGHT. HOWEVER, THIS SOFTWARE CODIFIES THE FINALIST
-- CANDIDATE ALGORITHMS (i.e., MARS, RC6tm, RIJNDAEL, SERPENT, AND
-- TWOFISH) IN THE ADVANCED ENCRYPTION STANDARD (AES) DEVELOPMENT EFFORT
-- SPONSORED BY THE NATIONAL INSTITUTE OF STANDARDS AND TECHNOLOGY (NIST)
-- AND MAY BE PROTECTED BY ONE OR MORE FORMS OF INTELLECTUAL PROPERTY. THE
-- U.S. GOVERNMENT MAKES NO WARRANTY, EITHER EXPRESSED OR IMPLIED,
-- INCLUDING BUT NO LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
-- OR FITNESS FOR A PARTICULAR PURPOSE, REGARDING THIS SOFTWARE. THE U.S.
-- GOVERNMENT FURTHER MAKES NO WARRANTY THAT THIS SOFTWARE WILL NOT
-- INFRINGE ANY OTHER UNITED STATES OR FOREIGN PATENT OR OTHER
-- INTELLECTUAL PROPERTY RIGHT. IN NO EVENT SHALL THE U.S. GOVERNMENT BE
-- LIABLE TO ANYONE FOR COMPENSATORY, PUNITIVE, EXEMPLARY, SPECIAL,
-- COLLATERAL, INCIDENTAL, CONSEQUENTIAL, OR ANY OTHER TYPE OF DAMAGES IN
-- CONNECTION WITH OR ARISING OUT OF COPY OR USE OF THIS SOFTWARE.
-- *************************************************************************
-- ===========================================================================
-- ===========================================================================
-- File Name : partial_kreg.vhdl
-- Author    : NSA
-- Date      : December 1999
-- Project   : AES Candidate Evaluation - MARS
-- Purpose   : This block stores the generated round keys for the pipelined 
--             case.  It dictates the motion of round keys through the
--             pipeline based on the latch signal.
-- Notes     : Used for pipeline case only.
-- ===========================================================================

library ieee;
use ieee.std_logic_1164.all;
use WORK.mars_pack.all;

-- ===========================================================================
-- =========================== Interface Description =========================
-- ===========================================================================

entity PARTIAL_TREG is

  port (clock    :  in std_logic;   -- clock signal
        reset    :  in std_logic;   -- active high reset (asynchronous)
    
        LATCH    :  in std_logic;
        DATA_IN  :  in T_ARRAY_TYPE;      -- input data bus

        DATA_OUT :  out T_ARRAY_TYPE      -- output data bus

  );

end PARTIAL_TREG;

architecture PARTIAL_TREG_RTL of PARTIAL_TREG is

-- ===========================================================================
-- =========================== Signal Definition =============================
-- ===========================================================================

signal DATA_OUT_INT : T_ARRAY_TYPE;

begin

-- ===========================================================================
-- =========================== Data Movement =================================
-- ===========================================================================

DATA_FLOW : process( clock, reset )

begin

   if reset = '1' then
      for index in 0 to 14 loop
          DATA_OUT_INT(index) <= ( others => '0' );
      end loop;
   

   elsif clock'event and clock = '1' then

      if LATCH = '1' then
        for index in 0 to 14 loop
          DATA_OUT_INT(index) <= DATA_IN(index);       -- latch input data
        end loop;
      else
        for index in 0 to 14 loop
          DATA_OUT_INT(index) <= DATA_OUT_INT(index);  -- hold data
        end loop;
      end if;

   end if;

end process; -- DATA_FLOW

end PARTIAL_TREG_RTL;
